//+------------------------------------------------------------------+
//|                                            CustomTicksJaffer.mq5 |
//|                        Copyright 2019, MetaQuotes Software Corp. |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2019, MetaQuotes Software Corp."
#property link      "https://www.mql5.com"
#property version   "1.00"
#property indicator_chart_window
#property indicator_plots 1
#include  <Includes\Symbol.mqh>

enum EnPriceTypes
  {
   Bid,
   Ask
  };
input datetime     InpStartDate   =D'2019.10.07 00:00:00'; //Start date
input int          InpChops       =10;                     //Chops
input EnPriceTypes InpAppliedPrice=0;                      //Price

int      ExtDataFilled=0 ;
string   ExtSymbolCustom=StringFormat("%s_%dsec",_Symbol,InpChops);
long     ExtChartId=0 ;
datetime ExtStart;
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
void OnInit()
  {
   ExtStart = InpStartDate - 60 ;
   int res = CustomTicksDelete(ExtSymbolCustom,0,LONG_MAX);
   Print("Ticks History Deleted : ",res,"  last_error ",GetLastError());
   ExtDataFilled=0;
//---
   const SYMBOL SYMB(ExtSymbolCustom);
   SYMB.CloneProperties(_Symbol);
   SYMB.On();
//---
   MqlTick ticks_add[];
   ResetLastError();
   res = CopyTicksRange(_Symbol,ticks_add, COPY_TICKS_INFO, ulong(InpStartDate)* 1000, ulong(TimeCurrent())*1000);
   Print("Ticks: ",res,"  last_error ",GetLastError());
   ExtStart = ExtStart - (res*60);
   Print("Start Date : ",ExtStart);
   for(int i=0 ; i<res && !IsStopped(); i++)
     {
      if(ExtDataFilled%InpChops==0)
        {
         ExtStart = ExtStart + 60 ;
         ExtStart = datetime(TimeToString(ExtStart, TIME_DATE) + " " + TimeToString(ExtStart, TIME_MINUTES) + ":00");
        }
      else
         ExtStart++;
      ticks_add[i].time = ExtStart;
      ticks_add[i].time_msc = long(ticks_add[i].time)*1000 ;
      ExtDataFilled++;
     }
//---
   int cnt = ArraySize(ticks_add);
   if(cnt>0)
     {
      ResetLastError();
      //res = CustomTicksAdd(ExtSymbolCustom,ticks_add);
      //Print("Ticks History Added : ",res,"  last_error ",GetLastError()," [",cnt,"]");
      res = CustomTicksReplace(ExtSymbolCustom,ticks_add[0].time_msc,ticks_add[cnt-1].time_msc,ticks_add);
      Print("Ticks History Replaced : ",res,"  last_error ",GetLastError()," [",cnt,"]");
     }
//---
   ResetLastError();
   ExtChartId = ChartOpen(ExtSymbolCustom,_Period);
   Print("ChartOpen  ",GetLastError());
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void OnDeinit(const int r)
  {
   if(ExtChartId != 0)
      ChartClose(ExtChartId);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
  {
//---
   if(rates_total<=0)
      return(0);
   MqlTick ticks[1];
   CopyTicks(_Symbol,ticks, COPY_TICKS_INFO, ulong(TimeCurrent())*1000, 1);
   if(ExtDataFilled%InpChops==0)
     {
      ExtStart = ExtStart+60 ;
      ExtStart = datetime(TimeToString(ExtStart, TIME_DATE) + " " + TimeToString(ExtStart, TIME_MINUTES) + ":00");
     }
   else
      ExtStart++;
   ticks[0].time = ExtStart;
   ticks[0].time_msc = long(ExtStart)*1000 ;
   ExtDataFilled++;

   ResetLastError();
   int res = CustomTicksAdd(ExtSymbolCustom,ticks);
   if(res<=0)
      Print(ExtStart,"  res=",res,"  last_error ",GetLastError());

//--- return value of prev_calculated for next call
   return(rates_total);
  }
//+------------------------------------------------------------------+
